import useHandleConversationStore from "@/feature/messenger/store/useHandleConversationStore";
import { createFullCalendarService, deleteFullCalendarService, editFullCalendarService, getFullCalendarService } from "@/feature/messenger/services/chat/FullCalendarService";
import useConfigChatStore from "@/feature/messenger/store/chat_store/useConfigChatStore";
import useConversationsStore from "@/feature/messenger/store/chat_store/useConversationsStore";
import { useEffect, useState } from "react";
import toast from 'react-hot-toast';

const useFollowUp = (): any => {

    //hooks
    const openModalFollowUp = useHandleConversationStore((state) => state.openModalFollowUp);
    const handleOpenModalFollowUp = useHandleConversationStore((state) => state.actions.handleOpenModalFollowUp);

    //stores
    const selectedConversation = useConversationsStore(state => state.selectedConversation);
    const defaultListConversation = useConversationsStore(state => state.defaultListConversation);
    const id_chatbot: string = useConfigChatStore(state => state.idChatbot);


    //states
    const [openFollowUpOptions, setOpenFollowUpOptions] = useState(false);
    const [typeOption, setTypeOption] = useState<string>('');
    const [loading, setLoading] = useState(false);
    const [dateSelected, setDateSelected] = useState('');
    const [listFollowUpEvents, setListFollowUpEvents] = useState([]);
    const [evenDataClick, setEvenDataClick] = useState<any>({});
    const [formData, setFormData] = useState({
        id: 0,
        commentary: '',
        sesions: '',
        color: '',
        date: ''
    })

    //functions
    const toggleFollowUp = () => {
        handleOpenModalFollowUp();
        //setOpenFollowUp(!openFollowUp);
    }
    const getAllEvents = async () => {
        const res = await getFullCalendarService(id_chatbot);
        if (res.success == true) {
            setListFollowUpEvents(res.data);
        }
    }

    const handleFormChange = (e: any) => {
        const { name, value } = e.target;
        setFormData({
            ...formData,
            [name]: value
        })
    }
    const openFollowUpNewEvent = (event: any) => {
        if (event.dateStr != undefined) {
            const date = event.dateStr.toString();
            setDateSelected(date);
            setOpenFollowUpOptions(!openFollowUpOptions);
            setTypeOption('add');
            setLoading(false);
            setFormData({
                id: 0,
                commentary: '',
                sesions: selectedConversation.sesion,
                color: '',
                date: date
            })
        } else {
            setEvenDataClick(event);
            setTypeOption('options');
            setLoading(false);
            setOpenFollowUpOptions(!openFollowUpOptions);
            setFormData({
                id: 0,
                commentary: event.event.title,
                sesions: '',
                color: '',
                date: ''
            })
        }

    }

    const handleCreateEvent = async (e: any) => {
        e.preventDefault();
        if (formData.commentary.trim() == '') {
            toast.error("Debe agregar un comentario");
            return
        }
        if (formData.sesions == undefined || formData.sesions.trim() == '') {
            toast.error("Debe seleccionar una sesión");
            return
        }
        /*if (formData.color.trim() == '') {
            toast.error("Debe seleccionar un color");
            return
        }*/
        if (typeOption == 'add') {
            setLoading(true);
            const res = await createFullCalendarService(id_chatbot, formData.commentary, formData.sesions, formData.color, dateSelected, dateSelected);
            if (res.success == true) {
                if (res.data != null) {
                    let list: any = [...listFollowUpEvents, res.data];
                    setListFollowUpEvents(list);
                }
                toast.success("Se ha creado el seguimiento correctamente");
                setLoading(false);
                setOpenFollowUpOptions(!openFollowUpOptions);
                return
            }
            setLoading(false);
            toast.error("Ocurrio un error al crear el seguimiento");
        } else if (typeOption == 'edit') {
            setLoading(true);
            const res = await editFullCalendarService(id_chatbot, formData.id.toString(), formData.commentary, formData.color, formData.date, formData.date);
            if (res.success == true) {
                if (res.data != null) {
                    let list: any = listFollowUpEvents.map((event: any) => {
                        if (event.id == formData.id) {
                            return res.data
                        }
                        return event
                    })
                    setListFollowUpEvents(list);
                }
                toast.success("Se ha actualizado el seguimiento correctamente");
                setLoading(false);
                setOpenFollowUpOptions(!openFollowUpOptions);
                return
            }
            setLoading(false);
            toast.error("Ocurrio un error al actualizar el seguimiento");
        }
    }
    const closeFollowUpNewEvent = () => {
        setOpenFollowUpOptions(!openFollowUpOptions);
    }

    const openFollowUpEditEvent = () => {
        const event = evenDataClick;
        const eventId = event.event.id;
        let eventData: any = listFollowUpEvents.find((event: any) => event.id == eventId);
        if (eventData != undefined) {
            setFormData({
                id: eventData.id,
                commentary: eventData.title,
                sesions: eventData.sesion,
                color: eventData.color,
                date: (eventData.start).split(" ")[0]
            })
            //setOpenFollowUpOptions(!openFollowUpOptions);
            setTypeOption('edit');
            setLoading(false);
            return
        }
        toast.error("Por el momento no se puede editar el seguimiento");
    }

    const openFollowUpDeleteEvent = () => {
        const event = evenDataClick;
        const eventId = event.event.id;
        let eventData: any = listFollowUpEvents.find((event: any) => event.id == eventId);
        if (eventData != undefined) {
            setFormData({
                id: eventData.id,
                commentary: eventData.title,
                sesions: eventData.sesion,
                color: eventData.color,
                date: (eventData.start).split(" ")[0]
            })
            setTypeOption('delete');
            setLoading(false);
            return
        }
        toast.error("Por el momento no se puede eliminar el seguimiento");
    }

    const onDropFollowUp = async (event: any) => {
        let newDate = event.event.start;
        let dateStart = (newDate.toISOString()).split("T")[0];
        const eventId = event.event.id;
        const eventData: any = listFollowUpEvents.find((event: any) => event.id == eventId);
        if (eventData != undefined) {
            setLoading(true);
            const res = await editFullCalendarService(id_chatbot, eventData.id.toString(), eventData.title, eventData.color, dateStart, dateStart);
        }
        setLoading(false);
    }

    const handleDeleteEvent = async (e: any) => {
        e.preventDefault();
        setLoading(true);
        const eventId = formData.id;
        if (eventId != 0) {
            const res = await deleteFullCalendarService(id_chatbot, eventId.toString());
            if (res.success == true && res.data == 1) {
                let list: any = listFollowUpEvents.filter((event: any) => event.id != eventId);
                setListFollowUpEvents(list);
                toast.success("Se ha eliminado el seguimiento correctamente");
                setLoading(false);
                setOpenFollowUpOptions(!openFollowUpOptions);
                return
            }
        }
        setLoading(false);
        toast.error("Por el momento no se puede eliminar el seguimiento");
    }

    useEffect(() => {
        if (openModalFollowUp) {
            getAllEvents();
        }
        /*const prevButton = document.querySelector('.fc-prev-button');
        if(prevButton){
            prevButton.innerHTML = `${<ChevronLeft size={16} />}`;
        }

        const nextButton = document.querySelector('.fc-next-button');
        if(nextButton){
            nextButton.innerHTML = `${<ChevronRight size={16} />}`;
        }*/
    }, [openModalFollowUp])


    return {
        openModalFollowUp,
        toggleFollowUp,
        openFollowUpOptions,
        listFollowUpEvents,
        loading,
        openFollowUpNewEvent,
        closeFollowUpNewEvent,
        onDropFollowUp,
        dateSelected,
        typeOption,
        handleCreateEvent,
        handleFormChange,
        formData,
        openFollowUpEditEvent,
        openFollowUpDeleteEvent,
        handleDeleteEvent,
        handleOpenModalFollowUp
    }
}

export default useFollowUp